from sms_pilot.exception import SMSValidationError


def is_valid_url(url):
    import re
    regex = re.compile(
        r'^https?://'
        r'(?:(?:[A-Z0-9](?:[A-Z0-9-]{0,61}[A-Z0-9])?\.)+[A-Z]{2,6}\.?|'
        r'localhost|'  # localhost...
        r'\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3})'
        r'(?::\d+)?'
        r'(?:/?|[/?]\S+)$', re.IGNORECASE)
    return url is not None and regex.search(url)


class Callback:

    def __init__(self, url, method: str = 'get'):
        self.url = url
        self.method = method

    def __validate(self):
        if not is_valid_url(self.url):
            raise SMSValidationError('Callback URL is invalid')
        if str(self.method).upper() not in ('GET', 'POST', 'PUT'):
            raise SMSValidationError('Callback method is invalid')

    def to_dict(self) -> dict:
        self.__validate()
        return dict(callback=self.url, callback_method=self.method)



import json
from datetime import datetime
from typing import Optional, Union, List

import requests
from .callback import Callback
import sms_pilot.objects as objects
from .exception import error_handle, SmsPilotAPIError


class SmsPilot:
    api_urls = {
        1: 'http://smspilot.ru/api.php',
        2: 'http://smspilot.ru/api2.php'
    }

    def __init__(self, api_key: str, default_sender: str = 'INFORM', callback: Callback = None, **options):
        """
        SmsPilot

        :param api_key: Ключ API
        :param default_sender: Отправитель по умолчанию
        :param callback: Колл-бэк
        :param options:
        :keyword test: Тестовый режим (без передачи оператору)
        :keyword cost: Получить только стоимость
        """
        self._callback = callback
        self._api_key = api_key
        self._default_sender = default_sender
        self.messages = []
        self._is_test = options.get('test', False)
        self._is_cost = options.get('cost', False)

    def prepare_request_data(self, data: dict):
        data.update(dict(
            apikey=self._api_key,
            format='json'
        ))
        if self._is_cost is not None:
            data['cost'] = int(self._is_cost)
        if self._is_test is not None:
            data['test'] = int(self._is_test)
        return data

    def _request(self, api_version: int, data: dict, method: str = 'POST') -> dict:
        try:
            api_url = self.api_urls[api_version]
        except KeyError:
            raise SmsPilotAPIError('Не верная версия API')
        headers = {
            'Content-type': 'application/json',
            'Accept': 'application/json',
            'User-Agent': 'SMSPilotPy/0.1'
        }
        if method == 'POST':
            response = requests.post(api_url, json=self.prepare_request_data(data), headers=headers)
        elif method == 'GET':
            response = requests.get(api_url, params=self.prepare_request_data(data), headers=headers)
        else:
            raise SmsPilotAPIError('Allowed GET or POST')
        if response.status_code != 200:
            raise SmsPilotAPIError('SomeError')

        try:
            response_data = response.json()
        except json.JSONDecodeError:
            raise SmsPilotAPIError(response.text)
        if 'error' in response_data:
            raise error_handle(response_data)

        return response_data

    def __get_last_message_id(self) -> int:
        return len(self.messages) + 1

    def send_message(self, to: Union[int, str], text: str, sender: str = None, **kwargs):
        """
        Отправить отдно сообщение

        :param to: Номер телефона
        :param text: Текст сообщения
        :param sender: Отправитель
        :param kwargs:
        :keyword callback Callback:
        :keyword ttl:
        :keyword send_datetime: Время отправки сообщения
        :return:
        """
        self.add_message(to, text, sender, **kwargs)
        return self.send_messages()

    def send_messages(self) -> objects.MessageResponse:
        """
        Отправить подготовленные сообщения

        :return:
        """
        data = {
            'send': self.messages
        }
        self.messages = []
        return objects.MessageResponse(self._request(2, data))

    def add_message(self, to: Union[str, int], text: str, sender: str = None, **kwargs):
        """
        Добавить сообщение в список

        :param to: Номер телефона
        :param text: Текст сообщения
        :param sender: Отправитель
        :param kwargs:
        :keyword callback Callback:
        :keyword ttl:
        :keyword send_datetime: Время отправки сообщения
        :return:
        """
        callback_obj = kwargs.get('callback', self._callback)
        time_to_live = kwargs.get('ttl')
        send_datetime = kwargs.get('send_datetime')
        msg = {
            'id': kwargs.get('id', self.__get_last_message_id()),
            'to': to,
            'from': sender or self._default_sender,
            'text': text
        }

        if isinstance(callback_obj, Callback):
            msg.update(callback_obj.to_dict())

        if isinstance(send_datetime, datetime):
            msg['send_datetime'] = send_datetime.strftime('%Y-%m-%d %H:%M:%S')

        if time_to_live:
            assert isinstance(time_to_live, int) and 1 <= time_to_live <= 1440, 'TTL может быть в диапозоне 1...1440'
            msg['ttl'] = time_to_live
        self.messages.append(msg)
        return self

    def check_by_server_id(self, server_ids: Union[int, List[int]]):
        if isinstance(server_ids, int):
            server_ids = [server_ids]
        data = {
            "check": {"server_id": v for v in server_ids}
        }
        return self._request(2, data)

    def check_by_server_pocket_id(self, server_pocket_id: int):
        data = {
            'check': True,
            'server_pocket_id': server_pocket_id
        }

        return objects.MessageCheckResponse(self._request(2, data))

    def get_balance(self, in_rur=True) -> Optional[float]:
        """
        Баланс

        :param in_rur: В рублях
        :return:
        """
        data = {
            'balance': 'rur' if in_rur else 'sms'
        }
        return self._request(2, data).get('balance')

    def user_info(self) -> objects.UserInfo:
        """
        Информация о пользователе

        :return:
        """
        return objects.UserInfo(self._request(2, dict(info=True)))

    def send_hlr(self, to: Union[str, int], callback: Callback = None) -> objects.MessageResponse:
        """
        HLR запрос

        :param to:
        :param callback:
        :return:
        """
        data = dict(send='HLR', to=to)
        if callback and isinstance(callback, Callback):
            data.update(callback.to_dict())
        return objects.MessageResponse(self._request(1, data, 'GET'))

    def ping(self, to: Union[str, int], callback: Callback = None) -> objects.MessageResponse:
        """
        PING запрос

        :param to:
        :param callback:
        :return:
        """
        data = dict(send='PING', to=to)
        if callback and isinstance(callback, Callback):
            data.update(callback.to_dict())
        return objects.MessageResponse(self._request(1, data, 'GET'))



class SmsPilotAPIError(Exception):
    pass


class ApiKeyBlocked(SmsPilotAPIError):
    pass


class ApiSystemError(SmsPilotAPIError):
    pass


class NoMoneyError(SmsPilotAPIError):
    pass


class SenderNotRegistered(SmsPilotAPIError):
    pass


class SMSValidationError(Exception):
    pass


errors_types = {
    106: ApiKeyBlocked,
    110: ApiSystemError,
    112: NoMoneyError,
    204: SenderNotRegistered
}


def error_handle(data: dict):
    error = data.get('error', {})
    error_code = int(error.get('code', 0))
    error_description = error.get('description')

    exc_class = errors_types.get(error_code, SmsPilotAPIError)

    return exc_class(f"{error_code}: {error_description}")


from setuptools import setup, find_packages
from os import path

this_directory = path.abspath(path.dirname(__file__))
with open(path.join(this_directory, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()

setup(
    name='sms-pilot-py',
    version='0.1',
    packages=find_packages(exclude=['tests*']),
    license='MIT',
    description='Python SMSPilot API',
    long_description=long_description,
    long_description_content_type='text/markdown',
    install_requires=['requests'],
    url='https://github.com/migelbd/SmsPilotPy',
    author='Mikhail Badrazhan',
    author_email='migel.bd@gmail.com'
)


from datetime import datetime

ERROR = -2
NOT_DELIVERED = -1
RECEIVED = 0
OPERATOR = 1
DELIVERED = 2
POSTPONED = 3

MESSAGE_STATUSES = (
    (ERROR, 'Ошибка', 'Ошибка, неправильные параметры запроса'),
    (NOT_DELIVERED, 'Не доставлено',
     'Сообщение не доставлено (не в сети, заблокирован, не взял трубку), PING - не в сети, HLR - не обслуживается (заблокирован)'),
    (RECEIVED, 'Новое', 'Новое сообщение/запрос, ожидает обработки у нас на сервере'),
    (OPERATOR, 'В очереди', 'Сообщение или запрос ожидают отправки на сервере оператора'),
    (DELIVERED, 'Доставлено', 'Доставлено, звонок совершен, PING - в сети, HLR - обслуживается'),
    (POSTPONED, 'Отложено', 'Отложенная отправка, отправка сообщения/запроса запланирована на другое время'),
)


def get_status_dict(short=True) -> dict:
    if short:
        return dict([(code, short) for code, short, description in MESSAGE_STATUSES])
    return dict([(code, description) for code, short, description in MESSAGE_STATUSES])


class Message:
    def __init__(self, **data):
        self.id = int(data.get('id'))
        self.server_id = int(data.get('server_id'))
        self.from_ = data.get('from')
        self.to = int(data.get('to'))
        self.text = data.get('text')
        self.parts = int(data.get('parts'))
        self.price = float(data.get('price'))
        self.status = int(data.get('status'))
        self.error = data.get('error')

        try:
            self.send_datetime = datetime.strptime(data.get('send_datetime'), '%Y-%m-%d %H:%M:%S')
        except ValueError:
            self.send_datetime = None

        self.country = data.get('country')
        self.operator = data.get('operator')

    def is_delivered(self) -> bool:
        return self.status == DELIVERED

    def is_error(self) -> bool:
        return self.status == ERROR

    def get_status_verbose(self) -> str:
        return get_status_dict().get(self.status)

    def __str__(self):
        return 'Message(%s): %s %s' % (self.id, self.to, self.get_status_verbose())

    def __repr__(self):
        return str(self)


class MessageResponse:

    def __init__(self, server_response: dict):
        self.send = [Message(**msg) for msg in server_response.get('send', [])]
        self.cost = float(server_response.get('cost', 0))
        self.balance = float(server_response.get('balance', 0))
        self.server_packet_id = int(server_response.get('server_packet_id', 0))

    def __len__(self):
        return len(self.send)

    def __str__(self):
        return 'MessageResponse: %s' % self.server_packet_id

    def __repr__(self):
        return str(self)


class MessageCheck:

    def __init__(self, check: dict):
        self.id = int(check.get('id', 0))
        self.server_id = int(check.get('id', 0))
        self.status = int(check.get('status', ERROR))
        self.modified = check.get('modified')

    def is_delivered(self) -> bool:
        return self.status == DELIVERED


class MessageCheckResponse:
    def __init__(self, server_response: dict):
        self.raw_response = server_response
        self.check = [MessageCheck(check) for check in server_response.get('check', [])]


class UserInfo:
    def __init__(self, server_response):
        info = server_response.get('info', {})
        self.id = int(info.get('id'))
        self.tariff_id = info.get('tariff_id')
        self.email = info.get('email')
        self.phone = info.get('phone')
        self.name = info.get('name')
        self.balance = info.get('balance')
        self.date = info.get('date')
        self.senders = info.get('senders')
        self.default_sender = info.get('default_sender')
        self.any_sender = info.get('any_sender')

    def __str__(self):
        return f'UserInfo: {self.name}'

    def __repr__(self):
        return str(self)
